/******************************************************************************
 * volume_hadow.h 
 * implements the construction and drawing of shadow volumes from triangulated surface meshes 
 * Copyright (C) 2010 Marc Gissler <gisslerm@informatik.uni-freiburg.de>
 ******************************************************************************/

#ifndef VOLUME_SHADOW_H
#define VOLUME_SHADOW_H

#include "shadow.h"
#include "mesh.h"

#include <vector>
#include <map>

#include "gmMath/gmVec3.h"
#include "gmMath/gmVec4.h"

// ----------------------------------------
// misc stuff
//

typedef gmVector4 ShadowVertex;

struct ShadowEdge
{
	// member variables
	CMesh::Edge edge;
	CMesh::Vertex h; // 3rd vertex from one of the faces this edge belongs to (the one that is lit)

	// operators
	friend bool operator<(const ShadowEdge& e0, const ShadowEdge& e1) { return e0.edge < e1.edge; }
	friend bool operator==(const ShadowEdge& e0, const ShadowEdge& e1){ return e0.edge == e1.edge; }

};

struct ShadowFace 
{
	// member variables
	ShadowVertex v1 ,v2, v3;

	// constructor
	ShadowFace(ShadowVertex& _v1 ,ShadowVertex& _v2, ShadowVertex& _v3) : v1(_v1), v2(_v2), v3(_v3) {}
};

class CVolumeShadowData : public CShadowData
{
public:
	std::vector<ShadowFace> m_vectorShadowFaces;
	std::vector<ShadowVertex> m_vectorShadowVertices;
};

// ------------------------------------------------------------
// defines thes shadow class
// 

class CVolumeShadow : public CShadow
{
public:
	// constructor/destructor
	CVolumeShadow();
	~CVolumeShadow();

	// builds the shadow volumes for this light
	void build(int lightId, gmVector3& lightPos);

	virtual void clear();

	// draws the scene with shadows
	virtual void drawSceneWithShadows();

	// member variables
private:
	
	// draws the shadow volumes
	void m_drawShadowVolumes(int lightId);
};

#endif
